# SWFFile - an SWF file object for zope
# Copyright 2003 Nick Evan-Wong


__version__ = 0.2

from OFS import PropertyManager, SimpleItem, FindSupport
from OFS.SimpleItem import Item
from OFS.FindSupport import FindSupport
from OFS.Image import Image
from OFS.PropertyManager import PropertyManager

from AccessControl import ClassSecurityInfo, getSecurityManager, Role
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Acquisition import Implicit
from Globals import Persistent
from Globals import InitializeClass
from Globals import MessageDialog
from AccessControl.Role import RoleManager
from Persistence import PersistentMapping

from ISWFFile import SWFFileInterface


# Management interface methods and declarations
# ---------------------------------------------


addSWFFileForm=PageTemplateFile('zpt/addSWF', globals())
editSWFFileForm=PageTemplateFile('zpt/editSWF', globals())

# ZMS constructor method
def manage_addSWFFile(
	self,
	id,
	title='',	
	file='',
	REQUEST=None
):
	"""ZMS constructor for the SWFFile class"""
		
	id=str(id)
	title=str(title)			
	
	self=self.this()
			
	self._setObject(id, SWFFile(id,title,file=file))
		
	if REQUEST is not None:		
        	REQUEST.RESPONSE.redirect('%s/manage_edit' % self._getOb(id).absolute_url())        	
        

# Content analyser functions borrowed from Zope product:
#
#    contentinfo
#    Copyright (C) 2002 Christoph Becker-Freyseng


# get unsigned bits
def getUBits(bitList, bitPos, nr):
    valD = 0
    for runD in xrange(bitPos, bitPos+nr):
	valD = valD*2 + bitList[runD]
    return valD

# get signed bits
def getSBits(bitList, bitPos, nr):
    valD = 0
    for runD in xrange(bitPos+1, bitPos+nr): #skip the first bit -> it is the sign
        valD = valD*2 + bitList[runD]
    if bitList[bitPos]: valD = -valD
    return valD

def analyseContent(data, filesize):
    
    if len(data)<=(8+17):
        raise "Content error","data too small!"
    # Uncompressed flash file
    if str(data[:3]) == 'FWS':
        compressed=0
    # ZLIB compressed flash file
    elif ((str(data[:3]) == 'CWS') or (str(data[:3]) == 'FWC')):
    	compressed=1
    else:
    	raise "Content Error","This does not appear to be an SWF file!"
    	
    version = str(ord(data[3]))
    size = ord(data[4]) + ord(data[5])*256 + ord(data[6])*256*256 + ord(data[7])*256*256*256
    
    bitList = []
    for runD in xrange(8, 8+17):
        byteD = ord(data[runD])
        bitList.append( (byteD&128)/128 )
        bitList.append( (byteD&64)/64 )
        bitList.append( (byteD&32)/32 )
        bitList.append( (byteD&16)/16 )
        bitList.append( (byteD&8)/8 )
        bitList.append( (byteD&4)/4 )
        bitList.append( (byteD&2)/2 )
        bitList.append( (byteD&1)/1 )
    bitPos = 0
    fieldSize = getUBits(bitList, bitPos, 5); bitPos=bitPos+ 5
    x1 = getSBits(bitList, bitPos, fieldSize); bitPos=bitPos+ fieldSize
    x2 = getSBits(bitList, bitPos, fieldSize); bitPos=bitPos+ fieldSize
    y1 = getSBits(bitList, bitPos, fieldSize); bitPos=bitPos+ fieldSize
    y2 = getSBits(bitList, bitPos, fieldSize); bitPos=bitPos+ fieldSize
    return FlashContent(filesize, version, compressed, x1, y1, x2, y2)


class FlashContent(Item):
    def __init__(self, filesize, version, compressed, flash_x1, flash_y1, flash_x2, flash_y2):
        self.mimetype = 'application/x-shockwave-flash'
        self.extension= '.swf'
        self.filesize= filesize
        self.flashversion= version or 0
        self.compressed= compressed
	self.flash_x1= flash_x1
	self.x1= int(flash_x1/20)
	self.flash_y1= flash_y1
	self.y1= int(flash_y1/20)
	self.flash_x2= flash_x2
	self.x2= int(flash_x2/20)
	self.flash_y2= flash_y2
	self.y2= int(flash_y2/20)
	
	self.width= int( (flash_x2 - flash_x1) /20)  
        self.height= int( (flash_y2 - flash_y1) /20) 


# Implement the SWFFile Class
# ---------------------------
class SWFFile(Image):
	"""Implementation of the SWFFile class"""
	
	meta_type='SWF File'

	__implements__ = SWFFileInterface

	# define properties
	getQuality = ['autolow','low','high','autohigh','best']
	getScale = ['showall','noborder','exactfit']
	getAlign = ['l','r','t','b']
	getWmode = ['window','opaque','transparent']

	_properties = (
			{'id':'title',			'type':'string',	'mode':'w'},
			{'id':'movieid', 		'type':'string',	'mode':'w'},
			{'id':'width', 			'type':'int', 		'mode':'w'},
	        {'id':'height', 		'type':'int', 		'mode':'w'},
	        {'id':'play',			'type':'boolean', 	'mode':'w'},
	        {'id':'loop', 			'type':'boolean', 	'mode':'w'},
	        {'id':'menu', 			'type':'boolean', 	'mode':'w'},
	        {'id':'swliveconnect', 	'type':'boolean', 	'mode':'w'},
	        {'id':'quality', 		'type':'selection', 'mode':'w', 'select_variable' : 'getQuality'},
	        {'id':'scale', 			'type':'selection', 'mode':'w',	'select_variable' : 'getScale'},
	        {'id':'align', 			'type':'selection', 'mode':'w', 'select_variable' : 'getAlign'},
	        {'id':'wmode', 			'type':'selection', 'mode':'w',	'select_variable' : 'getWmode'},
	        {'id':'bgcolor', 		'type':'string', 	'mode':'w'},
	        {'id':'version', 		'type':'string', 	'mode':''},
	        {'id':'content_type', 	'type':'string', 	'mode':''},
	        {'id':'compressed', 	'type':'boolean', 	'mode':''},
	)

	
	# define management templates	
	manage_edit=PageTemplateFile('zpt/editSWF', globals())

	manage_options=(
        	(        		
        		{'label':'SWF File', 'action':'manage_edit' },
        	)
		+ PropertyManager.manage_options
        	+ RoleManager.manage_options
        	+ Item.manage_options
        )
	
		
	
	__ac_permissions__=(
	('View management screens',
	('manage', 'manage_SWFFile', 'manage_SWFFiles')),
	('Change SWFFile',
	('manage_edit','manage_upload','PUT')),
 	('Manage SWF File properties', ('manage_addProperty',
                               'manage_editProperties',
                               'manage_delProperties',
                               'manage_changeProperties',
                               'manage_main',)),
        ('View',('index_html', 'view_image_or_file', 'get_size', 'getContentType', '')),
        ('FTP access', ('manage_FTPstat','manage_FTPget','manage_FTPlist')),
        ('Delete objects',('DELETE',)),
        )

	security=ClassSecurityInfo()

		

	def __init__(self, id,
		title='',
		movieid='',	
		play=1,
		loop=0,
		menu=1,
		swliveconnect=0,
		quality='high',
		scale='showall',
		wmode='window',
		align='',
		bgcolor='',
		file='',
		REQUEST=None):
		"initialise"
		self.__name__=id
        	self.title=title

		self.movieid = movieid
		self.play = play
		self.loop = loop
		self.menu = menu
		self.swliveconnect = swliveconnect
		self.quality = quality
		self.scale = scale
		self.wmode = wmode
		self.align = align
		self.bgcolor = bgcolor

		self.version = ''
		self.compressed = ''
		self.content_type = 'application/x-shockwave-flash'	
	
		# upload the SWF file if file is present
		if file:
			self.manage_upload(file=file)
        	
	def __str__(self):		
		"Default string representation of the SWFFile"
		return (self.tag())

   	
  	def __setstate__(self,state):
                Persistent.__setstate__(self,state)
                
                # 'autoplay' property renamed 'play' in version 0.2
                if not hasattr(self,'play'):
                	try: play = self.autoplay
                	except: play = 1
                	self.manage_addProperty('play','','boolean');    	    		
    	
		
 	def tag(self):        	
		"HTML tags for the SWF File"
		if self.loop: loop = 'true'
		else: loop = 'false'
		if self.menu: menu = 'true'
		else: menu = 'false'
		if self.swliveconnect: swliveconnect = 'true'
		else: swliveconnect = 'false'
		if self.play: play = 'true'
		else: play = 'false'
				
	        tag='<object classid=\"clsid:D27CDB6E-AE6D-11cf-96B8-444553540000\" codebase=\"http://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=6,0,0,0\" id=\"%s\" width=\"%s\" height=\"%s\" >\n' % (self.movieid,self.width, self.height)
	        tag+='\t<param name=\"movie\" value=\"'+self.absolute_url()+'\" />\n'
	        tag+='\t<param name=\"quality\" value=\"'+self.quality+'\" />\n'
	        tag+='\t<param name=\"scale\" value=\"'+self.scale+'\" />\n'
	        tag+='\t<param name=\"play\" value=\"'+play+'\" />\n'
	        tag+='\t<param name=\"loop\" value=\"'+loop+'\" />\n'
	        tag+='\t<param name=\"menu\" value=\"'+menu+'\" />\n'
	        tag+='\t<param name=\"swliveconnect\" value=\"'+swliveconnect+'\" />\n'
	        tag+='\t<param name=\"bgcolor\" value=\"'+self.bgcolor+'\" />\n'
	        tag+='\t<param name=\"align\" value=\"'+self.align+'\" />\n'
	        tag+='\t<param name=\"wmode\" value=\"'+self.wmode+'\" />\n'
	        tag+='<embed src=\"%s\" quality=\"%s\" bgcolor=\"%s\" width=\"%s\" height=\"%s\" name=\"%s\" align=\"%s\" type=\"application/x-shockwave-flash\" PLUGINSPAGE=\"http://www.macromedia.com/go/getflashplayer\"></embed>\n' % (self.absolute_url(), self.quality, self.bgcolor, self.width, self.height, self.movieid, self.align)
	        tag+='</object>\n'

        	return tag


	def manage_upload(self,file='',REQUEST=None):        	
		"Upload SWF file data"  	
		if self.wl_isLocked():
			raise ResourceLockedError, "File is locked via WebDAV"
				
		if not file: return

        	data, size = self._read_data(file)
        	
        	self.update_data(data, self.content_type, size)
		
		file.seek(0)
		data=file.read()
		filesize=len(data)
		analysed=analyseContent(data,filesize)

		# set analysed property values
		o=self		
		o.height=analysed.height
		o.width=analysed.width
		o.version=analysed.flashversion
		o.compressed=analysed.compressed

        	if REQUEST:
            		message="Saved changes."
            		return self.manage_edit(self,REQUEST,manage_tabs_message=message)
            				
    	


# Initialize product class
# ------------------------

InitializeClass(SWFFile)
